{% if theme_logo %}
<p class="logo">
    <a href="{{ pathto(master_doc) }}">
        <img class="logo" src="{{ pathto('_static/' ~ theme_logo, 1) }}" alt="Logo"/>
        {% if theme_logo_name|lower == 'true' %}
        <h1 class="logo logo-name">{{ project }}</h1>
        {% endif %}
    </a>
</p>
{% else %}
<h1 class="logo"><a href="{{ pathto(master_doc) }}">{{ project }}</a></h1>
{% endif %}

{% if theme_description %}
<p class="blurb">{{ theme_description }}</p>
{% endif %}

{% macro badge(badge, href) -%}
<p><a href="{{href}}"><img src="https://img.shields.io/{{badge}}"/></a></p>
{%- endmacro %}

{% if theme_github_user and theme_github_repo and not flycheck_offline_html %}
{% if theme_github_button|lower == 'true' %}
<p>
    <iframe src="https://ghbtns.com/github-btn.html?user={{ theme_github_user }}&amp;repo={{ theme_github_repo }}&amp;type={{ theme_github_type }}&amp;count={{ theme_github_count }}&amp;v=2"
            allowtransparency="true" frameborder="0" scrolling="0" width="170px" height="20px"></iframe>
</p>
{% endif %}
{% endif %}

{% if not flycheck_offline_html %}
<p>
    <a href="https://gitter.im/flycheck/flycheck">
        <img src="https://img.shields.io/gitter/room/flycheck/flycheck.svg?maxAge=2592000" />
    </a>
</p>
<p>
    <a href="https://stable.melpa.org/#/flycheck"><img alt="MELPA Stable" src="https://stable.melpa.org/packages/flycheck-badge.svg"/></a>
</p>
<p>
    <a href="https://melpa.org/#/flycheck"><img alt="MELPA" src="https://melpa.org/packages/flycheck-badge.svg"/></a>
</p>
{% endif %}
