<h4>Tables</h4>
<ul>
    <li><a href="{{pathto('languages')}}">Supported languages</a></li>
    <li><a href="{{pathto('changes')}}">List of changes</a></li>
    <li><a href="{{pathto('genindex')}}">Index</a></li>
</ul>
