;;; dummy-package-autoloads.el --- automatically extracted autoloads
;;
;;; Code:


;;;### (autoloads nil "dummy-package" "dummy-package.el" (20978 39905
;;;;;;  0 0))
;;; Generated autoloads from dummy-package.el

(autoload 'dummy-package-foo "dummy-package" "\


\(fn)" nil nil)

;;;***

;;;### (autoloads nil nil ("dummy-package-pkg.el") (20978 39909 438930
;;;;;;  0))

;;;***

(provide 'dummy-package-autoloads)
;; Local Variables:
;; version-control: never
;; no-byte-compile: t
;; no-update-autoloads: t
;; coding: utf-8
;; End:
;;; dummy-package-autoloads.el ends here
